package de.tudresden.inf.st.mquat.solving.ilp;

import de.tudresden.inf.st.mquat.jastadd.model.*;
import de.tudresden.inf.st.mquat.solving.clustering.KMeans;

import java.util.ArrayList;
import java.util.Map;

public class AbstractILPTransformer implements ILPTransformer {

  private int numClusters = 0;

  public AbstractILPTransformer() {
    // default constructor
  }

  public AbstractILPTransformer(int numClusters) {
    this.numClusters = numClusters;
  }

  @Override
  public ILP getILP(Root model) {
    return model.getAbstractILP();
  }

  @Override
  public String getSuffix() {
    return "-abstract";
  }

  @Override
  public void prepareModel(Root model) {
    if (this.numClusters == 0) {
      numClusters = Math.max(2,model.getHardwareModel().getNumResource()/5);
    }
    KMeans kmeans = new KMeans(model);

    Map<Integer, ArrayList<Resource>> clusters = kmeans.buildClusters(numClusters);

    for (int clusterId : clusters.keySet()) {

      if (clusters.get(clusterId).size() > 0) {
        ResourceClass resourceClass = new ResourceClass();
        resourceClass.setName(new Name("class_" + clusterId));

        for (Resource resource : clusters.get(clusterId)) {
          resource.setResourceClassRef(resourceClass.createRef());
          resourceClass.addResourceRef(resource.createRef());
        }

        model.getHardwareModel().addResourceClass(resourceClass);
      }

    }
  }


}
