package de.tudresden.inf.st.mquat.solving;

import beaver.Parser;
import de.tudresden.inf.st.mquat.jastadd.model.*;
import de.tudresden.inf.st.mquat.solving.ilp.AbstractILPSolver;
import de.tudresden.inf.st.mquat.solving.ilp.AbstractILPTransformer;
import de.tudresden.inf.st.mquat.utils.ParserUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Assert;
import org.junit.Test;

import java.io.IOException;

public abstract class AbstractHandwrittenTestSuite extends ILPHandwrittenTest {

  private static Logger logger = LogManager.getLogger(AbstractHandwrittenTestSuite.class);

  protected Tuple<Root, Solution> loadAndSolve(String filename, int numClusters) throws IOException, Parser.Exception, SolvingException {
    Root model = ParserUtils.load(filename, HandwrittenTestSuite.class);
    AbstractILPSolver solver = (AbstractILPSolver) getSolver();
    solver.setTransformer(new AbstractILPTransformer(numClusters));
    Solution solution = solver.solve(model);

    if (logger.isDebugEnabled()) {
      MquatString out = solution.print(new MquatWriteSettings(" "));
      logger.debug("Solution:\n{}", out);
    }
    return new Tuple<>(model, solution);
  }

  /**
   * Create and return the solver to use in this test suite
   * @return a solver instance
   */
  protected abstract Solver getSolver();

  protected Tuple<Root, Solution> loadAndSolve(String filename) throws IOException, Parser.Exception, SolvingException {
    Root model = ParserUtils.load(filename, HandwrittenTestSuite.class);
    Solution solution = getSolver().solve(model);
    if (logger.isDebugEnabled()) {
      MquatString out = solution.print(new MquatWriteSettings(" "));
      logger.debug("Solution:\n{}", out);
    }
    return new Tuple<>(model, solution);
  }

  @Test
  public void test_01() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_01.txt");
    assertValidSolution(modelAndSolution);
    Assignment config_0i0m0 = assertAssignment(modelAndSolution, 0, "config_0i0m0", "r0");
    Assignment config_1i0m0 = assertAssignment(modelAndSolution, 0, "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
  }

  @Test
  public void test_01_abstract() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_01_abstract.txt", 2);
    assertValidSolution(modelAndSolution);
    Assignment config_0i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_0i0m0", "r0");
    Assignment config_1i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
  }

  @Test
  public void test_02() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_02.txt");
    assertValidSolution(modelAndSolution);
    Assignment config_0i0m0 = assertAssignment(modelAndSolution, 0, "config_0i0m0", "r0");
    Assignment config_1i0m0 = assertAssignment(modelAndSolution, 0, "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
  }

  @Test
  public void test_02_abstract() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_02_abstract.txt", 2);
    assertValidSolution(modelAndSolution);
    Assignment config_0i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_0i0m0", "r0");
    Assignment config_1i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
  }

  @Test
  public void test_02_abstract_2() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_02_abstract_2.txt", 2);
    assertValidSolution(modelAndSolution);
    Assignment config_0i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_0i0m0", "r0");
    Assignment config_1i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq1", "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
    config_0i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq2", "config_0i0m0", "r0");
    config_1i0m0 = assertAssignment(modelAndSolution.getSecondElement(), "rq2", "config_1i0m0", "r1");
    assertComponentRequirement(config_0i0m0,"other", config_1i0m0);
  }

  @Test
  public void test_03_abstract_1() throws IOException, Parser.Exception, SolvingException {
    Tuple<Root, Solution> modelAndSolution = loadAndSolve("test_03_abstract_1.txt", 1);
    Assert.assertFalse("solution is invalid.", modelAndSolution.getSecondElement().isValid());
    Assert.assertTrue("solution is empty.", modelAndSolution.getSecondElement().getNumAssignment() == 0);
  }

}
