package de.tudresden.inf.st.mquat.solving;

import de.tudresden.inf.st.mquat.generator.ScenarioDescription;
import de.tudresden.inf.st.mquat.generator.ScenarioGenerator;
import de.tudresden.inf.st.mquat.jastadd.model.Root;
import de.tudresden.inf.st.mquat.jastadd.model.Solution;
import de.tudresden.inf.st.mquat.solving.ilp.AbstractILPTransformer;
import de.tudresden.inf.st.mquat.solving.ilp.GLPKSolver;
import de.tudresden.inf.st.mquat.solving.ilp.ILPExternalSolver;
import de.tudresden.inf.st.mquat.solving.ilp.SCIPSolver;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.concurrent.TimeUnit;

public class AbstractILPTest1 {

  private static Logger logger;

  @BeforeClass
  public static void initLogger() {
    logger = LogManager.getLogger(AbstractILPTest1.class);
  }



  @Test
  public void test_config_01() throws SolvingException {
    int tlc = 1;
    int iac = 1;
    int isd = 0;
    int cac = 0;
    int csd = 0;
    int dep = 4;
    int imp = 4;
    int res = 80;
    int req = 2;
    int cpu = 1;
    int seed = 0;

    StringBuilder report = new StringBuilder();
    report.append("\n");

    ScenarioGenerator generator;
    Root model;
    ILPExternalSolver solver;
    Solution solution;

    generator = new ScenarioGenerator(new ScenarioDescription(tlc, iac, isd, cac, csd, dep, imp, res, req, cpu, seed));
    model = generator.generate();
    solver = new SCIPSolver();
    solver.setDeleteFilesOnExit(false).setTimeout(15, TimeUnit.MINUTES);
    solution = solver.solve(model);
    report.append("Concrete (valid=").append(solution.isValid()).append(", objective=").append(solution.computeObjective()).append(") in " + solver.getLastGenerationTime() + " + " + solver.getLastSolvingTime() + "\n");


    for (int numClusters = 80; numClusters <= 640; numClusters+=80) {
      model = generator.generate();
      solver = new SCIPSolver(new AbstractILPTransformer(numClusters));
      solver.setDeleteFilesOnExit(false).setTimeout(5, TimeUnit.MINUTES);
      solution = solver.solve(model);
      report.append("Abstract with "+ numClusters +" clusters (valid=").append(solution.isValid()).append(", objective=").append(solution.computeObjective()).append(") in " + solver.getLastGenerationTime() + " + " + solver.getLastSolvingTime() + "\n");
    }


    logger.info(report.toString());
  }
}
